#include <stdlib.h>

#include "mathtable.h"

/* declare math function type */
typedef int (*math2_t)(int,int);
typedef int (*math4_t)(int,int,int,int);

/* function table entry */
typedef struct _math_func_info {
    union { /* to avoid type check error */
        math2_t math2;
        math4_t math4;
    } func;
    int nOfArgs;
} math_func_info_t;

/* actual implementations */
static int add(int a, int b) {
    return a + b;
} 

static int sub(int a, int b) {
    return a - b;
}

static int mult(int a, int b) {
    return a * b;
}

static int slope(int u, int v, int x, int y ) {
    return (v-y)/(u-x);
}

/* again, this is GCC specific way to initialize structure */
static math_func_info_t funcTable[MAX_FUNC_ID] = {
    { .func.math2 = add, .nOfArgs = 2 },
    { .func.math2 = sub, .nOfArgs = 2 },
    { .func.math2 = mult, .nOfArgs = 2 },
    { .func.math4 = slope, .nOfArgs = 4 },
};

/* the only exposed interface */
int math_call(mathfunctype_t id, const matharg_t *arg,int *res)
{
    math_func_info_t *funcInfo;

    /* check parameters */
    if ( id < 0 || id >= MAX_FUNC_ID || arg == NULL || res == NULL )
        return -1;

    funcInfo = &funcTable[id];
    /* check the number of arguments */
    if ( funcInfo->nOfArgs != arg->argc )
        return -1;

    switch ( funcInfo->nOfArgs ) {
        case 2:
            *res = funcInfo->func.math2(arg->arg.arg2[0],arg->arg.arg2[1]);
            break;
        case 4:
            *res = funcInfo->func.math4(arg->arg.arg4[0],arg->arg.arg4[1],arg->arg.arg4[2],arg->arg.arg4[3]);
            break;
        default:
            return -1;
    }

    return 0;
}

